@@ -1068,9 +1068,10 @@ mod tests {
                 unreachable!("Schnorr sig not tested in this test")
             }
         };
-
+        let txtmpl_hash: &sha256::Hash = &sha256::Hash::from_inner([34u8; 32]);
         fn from_stack<'txin, 'elem>(
             verify_fn: Box<dyn FnMut(&KeySigPair) -> bool + 'elem>,
+            txtmpl_hash: &'elem sha256::Hash,
             stack: Stack<'txin>,
             ms: &'elem Miniscript<BitcoinKey, NoChecks>,
         ) -> Iter<'elem, 'txin> {
@@ -1086,6 +1087,8 @@ mod tests {
                 age: 1002,
                 height: 1002,
                 has_errored: false,
+                // TODO: Replace with actual hash
+                txtemplate: &txtmpl_hash,
             }
         }
 
@@ -1107,10 +1110,11 @@ mod tests {
         let hash160 = no_checks_ms(&format!("hash160({})", hash160_hash));
         let ripemd160_hash = ripemd160::Hash::hash(&preimage);
         let ripemd160 = no_checks_ms(&format!("ripemd160({})", ripemd160_hash));
+        let txtemplate = no_checks_ms(&format!("t:txtmpl({})", txtmpl_hash));
 
         let stack = Stack::from(vec![stack::Element::Push(&der_sigs[0])]);
         let vfyfn = vfyfn_.clone(); // sigh rust 1.29...
-        let constraints = from_stack(Box::new(vfyfn), stack, &pk);
+        let constraints = from_stack(Box::new(vfyfn), txtmpl_hash, stack, &pk);
         let pk_satisfied: Result<Vec<SatisfiedConstraint>, Error> = constraints.collect();
         assert_eq!(
             pk_satisfied.unwrap(),
@@ -1122,7 +1126,7 @@ mod tests {
         //Check Pk failure with wrong signature
         let stack = Stack::from(vec![stack::Element::Dissatisfied]);
         let vfyfn = vfyfn_.clone(); // sigh rust 1.29...
-        let constraints = from_stack(Box::new(vfyfn), stack, &pk);
+        let constraints = from_stack(Box::new(vfyfn), txtmpl_hash, stack, &pk);
         let pk_err: Result<Vec<SatisfiedConstraint>, Error> = constraints.collect();
         assert!(pk_err.is_err());
 
@@ -1133,7 +1137,7 @@ mod tests {
             stack::Element::Push(&pk_bytes),
         ]);
         let vfyfn = vfyfn_.clone(); // sigh rust 1.29...
-        let constraints = from_stack(Box::new(vfyfn), stack, &pkh);
+        let constraints = from_stack(Box::new(vfyfn), txtmpl_hash, stack, &pkh);
         let pkh_satisfied: Result<Vec<SatisfiedConstraint>, Error> = constraints.collect();
         assert_eq!(
             pkh_satisfied.unwrap(),
@@ -1146,7 +1150,7 @@ mod tests {
         //Check After
         let stack = Stack::from(vec![]);
         let vfyfn = vfyfn_.clone(); // sigh rust 1.29...
-        let constraints = from_stack(Box::new(vfyfn), stack, &after);
+        let constraints = from_stack(Box::new(vfyfn), txtmpl_hash, stack, &after);
         let after_satisfied: Result<Vec<SatisfiedConstraint>, Error> = constraints.collect();
         assert_eq!(
             after_satisfied.unwrap(),
@@ -1156,7 +1160,7 @@ mod tests {
         //Check Older
         let stack = Stack::from(vec![]);
         let vfyfn = vfyfn_.clone(); // sigh rust 1.29...
-        let constraints = from_stack(Box::new(vfyfn), stack, &older);
+        let constraints = from_stack(Box::new(vfyfn), txtmpl_hash, stack, &older);
         let older_satisfied: Result<Vec<SatisfiedConstraint>, Error> = constraints.collect();
         assert_eq!(
             older_satisfied.unwrap(),
@@ -1166,7 +1170,7 @@ mod tests {
         //Check Sha256
         let stack = Stack::from(vec![stack::Element::Push(&preimage)]);
         let vfyfn = vfyfn_.clone(); // sigh rust 1.29...
-        let constraints = from_stack(Box::new(vfyfn), stack, &sha256);
+        let constraints = from_stack(Box::new(vfyfn), txtmpl_hash, stack, &sha256);
         let sah256_satisfied: Result<Vec<SatisfiedConstraint>, Error> = constraints.collect();
         assert_eq!(
             sah256_satisfied.unwrap(),
@@ -1179,7 +1183,7 @@ mod tests {
         //Check Shad256
         let stack = Stack::from(vec![stack::Element::Push(&preimage)]);
         let vfyfn = vfyfn_.clone(); // sigh rust 1.29...
-        let constraints = from_stack(Box::new(vfyfn), stack, &hash256);
+        let constraints = from_stack(Box::new(vfyfn), txtmpl_hash, stack, &hash256);
         let sha256d_satisfied: Result<Vec<SatisfiedConstraint>, Error> = constraints.collect();
         assert_eq!(
             sha256d_satisfied.unwrap(),
@@ -1192,7 +1196,7 @@ mod tests {
         //Check hash160
         let stack = Stack::from(vec![stack::Element::Push(&preimage)]);
         let vfyfn = vfyfn_.clone(); // sigh rust 1.29...
-        let constraints = from_stack(Box::new(vfyfn), stack, &hash160);
+        let constraints = from_stack(Box::new(vfyfn), txtmpl_hash, stack, &hash160);
         let hash160_satisfied: Result<Vec<SatisfiedConstraint>, Error> = constraints.collect();
         assert_eq!(
             hash160_satisfied.unwrap(),
@@ -1205,7 +1209,7 @@ mod tests {
         //Check ripemd160
         let stack = Stack::from(vec![stack::Element::Push(&preimage)]);
         let vfyfn = vfyfn_.clone(); // sigh rust 1.29...
-        let constraints = from_stack(Box::new(vfyfn), stack, &ripemd160);
+        let constraints = from_stack(Box::new(vfyfn), txtmpl_hash, stack, &ripemd160);
         let ripemd160_satisfied: Result<Vec<SatisfiedConstraint>, Error> = constraints.collect();
         assert_eq!(
             ripemd160_satisfied.unwrap(),
@@ -1215,6 +1219,18 @@ mod tests {
             }]
         );
 
+        //Check txtemplate
+        let stack = Stack::from(vec![]);
+        let vfyfn = vfyfn_.clone(); // sigh rust 1.29...
+        let constraints = from_stack(Box::new(vfyfn), txtmpl_hash, stack, &txtemplate);
+        let txtemplate_satisfied: Result<Vec<SatisfiedConstraint>, Error> = constraints.collect();
+        assert_eq!(
+            txtemplate_satisfied.unwrap(),
+            vec![SatisfiedConstraint::TxTemplate {
+                hash: txtmpl_hash.clone()
+            }]
+        );
+
         //Check AndV
         let pk_bytes = pks[1].to_public_key().to_bytes();
         let stack = Stack::from(vec![
@@ -1228,7 +1244,7 @@ mod tests {
             pks[1].to_pubkeyhash()
         ));
         let vfyfn = vfyfn_.clone(); // sigh rust 1.29...
-        let constraints = from_stack(Box::new(vfyfn), stack, &elem);
+        let constraints = from_stack(Box::new(vfyfn), txtmpl_hash, stack, &elem);
 
         let and_v_satisfied: Result<Vec<SatisfiedConstraint>, Error> = constraints.collect();
         assert_eq!(
@@ -1254,7 +1270,7 @@ mod tests {
             pks[0], sha256_hash
         ));
         let vfyfn = vfyfn_.clone(); // sigh rust 1.29...
-        let constraints = from_stack(Box::new(vfyfn), stack, &elem);
+        let constraints = from_stack(Box::new(vfyfn), txtmpl_hash, stack, &elem);
 
         let and_b_satisfied: Result<Vec<SatisfiedConstraint>, Error> = constraints.collect();
         assert_eq!(
@@ -1282,7 +1298,7 @@ mod tests {
             pks[1].to_pubkeyhash(),
         ));
         let vfyfn = vfyfn_.clone(); // sigh rust 1.29...
-        let constraints = from_stack(Box::new(vfyfn), stack, &elem);
+        let constraints = from_stack(Box::new(vfyfn), txtmpl_hash, stack, &elem);
 
         let and_or_satisfied: Result<Vec<SatisfiedConstraint>, Error> = constraints.collect();
         assert_eq!(
@@ -1306,7 +1322,7 @@ mod tests {
             stack::Element::Dissatisfied,
         ]);
         let vfyfn = vfyfn_.clone(); // sigh rust 1.29...
-        let constraints = from_stack(Box::new(vfyfn), stack, &elem);
+        let constraints = from_stack(Box::new(vfyfn), txtmpl_hash, stack, &elem);
 
         let and_or_satisfied: Result<Vec<SatisfiedConstraint>, Error> = constraints.collect();
         assert_eq!(
@@ -1327,7 +1343,7 @@ mod tests {
             pks[0], sha256_hash
         ));
         let vfyfn = vfyfn_.clone(); // sigh rust 1.29...
-        let constraints = from_stack(Box::new(vfyfn), stack, &elem);
+        let constraints = from_stack(Box::new(vfyfn), txtmpl_hash, stack, &elem);
 
         let or_b_satisfied: Result<Vec<SatisfiedConstraint>, Error> = constraints.collect();
         assert_eq!(
@@ -1345,7 +1361,7 @@ mod tests {
             pks[0], sha256_hash
         ));
         let vfyfn = vfyfn_.clone(); // sigh rust 1.29...
-        let constraints = from_stack(Box::new(vfyfn), stack, &elem);
+        let constraints = from_stack(Box::new(vfyfn), txtmpl_hash, stack, &elem);
 
         let or_d_satisfied: Result<Vec<SatisfiedConstraint>, Error> = constraints.collect();
         assert_eq!(
@@ -1365,7 +1381,7 @@ mod tests {
             sha256_hash, pks[0]
         ));
         let vfyfn = vfyfn_.clone(); // sigh rust 1.29...
-        let constraints = from_stack(Box::new(vfyfn), stack, &elem);
+        let constraints = from_stack(Box::new(vfyfn), txtmpl_hash, stack, &elem);
 
         let or_c_satisfied: Result<Vec<SatisfiedConstraint>, Error> = constraints.collect();
         assert_eq!(
@@ -1385,7 +1401,7 @@ mod tests {
             sha256_hash, pks[0]
         ));
         let vfyfn = vfyfn_.clone(); // sigh rust 1.29...
-        let constraints = from_stack(Box::new(vfyfn), stack, &elem);
+        let constraints = from_stack(Box::new(vfyfn), txtmpl_hash, stack, &elem);
 
         let or_i_satisfied: Result<Vec<SatisfiedConstraint>, Error> = constraints.collect();
         assert_eq!(
@@ -1408,7 +1424,7 @@ mod tests {
             pks[4], pks[3], pks[2], pks[1], pks[0],
         ));
         let vfyfn = vfyfn_.clone(); // sigh rust 1.29...
-        let constraints = from_stack(Box::new(vfyfn), stack, &elem);
+        let constraints = from_stack(Box::new(vfyfn), txtmpl_hash, stack, &elem);
 
         let thresh_satisfied: Result<Vec<SatisfiedConstraint>, Error> = constraints.collect();
         assert_eq!(
@@ -1438,7 +1454,7 @@ mod tests {
             pks[4], pks[3], pks[2], pks[1], pks[0],
         ));
         let vfyfn = vfyfn_.clone(); // sigh rust 1.29...
-        let constraints = from_stack(Box::new(vfyfn), stack, &elem);
+        let constraints = from_stack(Box::new(vfyfn), txtmpl_hash, stack, &elem);
 
         let multi_satisfied: Result<Vec<SatisfiedConstraint>, Error> = constraints.collect();
         assert_eq!(
@@ -1468,7 +1484,7 @@ mod tests {
             pks[4], pks[3], pks[2], pks[1], pks[0],
         ));
         let vfyfn = vfyfn_.clone(); // sigh rust 1.29...
-        let constraints = from_stack(Box::new(vfyfn), stack, &elem);
+        let constraints = from_stack(Box::new(vfyfn), txtmpl_hash, stack, &elem);
 
         let multi_error: Result<Vec<SatisfiedConstraint>, Error> = constraints.collect();
         assert!(multi_error.is_err());
